@@ -81,7 +81,7 @@ A Hash class compatible with NetCDF attributes.
 	    end
 	    self._set_(key, val)
 	 else
-	    raise ArgumentError, "Not allowed as an attribute value: #{val}" 
+	    raise ArgumentError, "Not allowed as an attribute value: #{val} (String or NArray/Array of numerics are required`)" 
 	 end
 	 val
       end
