@@ -1,6 +1,7 @@
 require "numru/gphys/varray"
 require "numru/gphys/varraynetcdf"
 require "numru/gphys/varraycomposite"
+require "date"
 
 =begin
 =class NumRu::Axis
@@ -46,6 +47,12 @@ simulation. The 3rd is not physical and may not be considered as
     RETURN VALUE
     * an Array of String. Current default value is ["integrate","average"].
 
+---Axis.humane_messaging = t_or_f
+    If false is given, the [] method is changed to return a naive 
+    straightforward message if the axis is lost. -- By default,
+    it returns a fancy but sometimes unwanted message if the 
+    axis is the time with since field in the form of yyyy-mm-dd....    
+
 ==Instance Methods
 ---name=(nm)
     Set a name.
@@ -316,6 +323,12 @@ module NumRu
 
    class Axis
 
+      @@humane_message = true  # if true, [] returns a humane message when the
+                               # axis is lost. if false, it returns a naive one
+      def self.humane_message=(t_or_f)
+	@@humane_message = t_or_f
+      end
+
       def initialize(cell=false,bare_index=false,name=nil)
 	 @init_fin = false         # true/false (true if initializatn finished)
 	 @name = name              # String
@@ -549,10 +562,39 @@ module NumRu
 	 when Fixnum
 	    pos=@pos[slicer]
 	    cval = ( pos.val.is_a?(Numeric) ? pos.val : pos.val[0] )
-	    sval = sprintf("%g",cval)
-	    info_lost = "#{self.name}=#{sval}"
-	    if (units = pos.get_att('units'))   # substitution
-	      info_lost += " "+units
+	    units = pos.get_att('units')
+	    info_lost = false
+	    if @@humane_message
+	      if units && /(.*) *since *(.*)/ =~ units  
+		stun = $1
+		tun = Units[stun]
+		since = DateTime.parse($2)
+		if Units[units] =~ Units['days since 1-1-1'] # s,hour,day,..
+		  sec = tun.convert( cval, Units['seconds'] ).round + 1e-1
+			 # ^ take "round" to avoid round error in sec
+			 # (Note that %S below takes floor of seconds).
+		  datetime = since + (sec/86400.0)
+		  info_lost = datetime.strftime('%Y-%m-%d %H:%M:%S%z')
+		  info_lost_set = true
+		elsif Units[units] =~ Units['months since 1-1-1'] # year,month,pentad,..
+		  monun = Units['months']
+		  if /mon/ =~ stun || /year/ =~ stun
+		    datetime = since >> tun.convert( cval, monun ).round
+		    info_lost = datetime.strftime('%Y-%m')
+		  else  # maybe pentad
+		    datetime = since >> tun.convert( cval, monun )
+		    info_lost = datetime.strftime('%Y-%m-%d')
+		  end
+		  info_lost_set = true
+		end
+	      end
+	    end
+	    if !info_lost_set
+	      sval = sprintf("%g",cval)
+	      info_lost = "#{self.name}=#{sval}"
+	      if (units)   # substitution
+		info_lost += " "+units
+	      end
 	    end
 	    return info_lost
 	 when Range, true
@@ -643,7 +685,7 @@ module NumRu
 	  xmax = range_ary.max
 	  wh = ( (ax >= xmin) & (ax <= xmax) ).where
 	  if wh.length == 0
-	    raise "Range #{range} does not include any grid point of #{dim}-th axis"
+	    raise "Range #{range} does not include any grid point in the axis '#{name}'"
 	  end
 	  idmin = wh.min
 	  idmax = wh.max
