@@ -594,7 +594,11 @@ module NumRu
          eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{method}(dim_or_dimname, *extra_args)
             vary, grid = @grid.#{method}(@data, dim_or_dimname, *extra_args)
-	    GPhys.new( grid, vary )
+	    if grid 
+	      GPhys.new( grid, vary )
+	    else
+	      vary    # scalar
+	    end
 	 end
 	 EOS
       end
