@@ -10,17 +10,9 @@ through which the users can also access its binary data file(s).
 
 ==Current Limitations
 
-* The "units" field in variable definitions are only partially interpreted.
-  Interpreted are 99 or 0 (default), and -1,30.
 * option 365_day_calendar is not interpreted
-* Partial support of possible units of the time increment:
-  mn (minutes), hr (hours) and dy (days) are supported, but
-  mo (month) and yr (year) is not. 
-    (Developer's memo): Time is stored as Julian day and is handled with
-    the Date standard library. Thus, it's not difficult to support the
-    unsupported units. However, it would be tricky to support the
-    365_day_calendar option.
-* Partial support of the "template" option (only %m2,%y2,%y4) 
+* Partial support of the "template" option 
+  (only %y2,%y4,%m1,%m2,%d1,%d2,%h1,%h2,%n2) 
   (Time is assumed to be increasing monotonically).
 
 ==Class Methods
@@ -116,15 +108,15 @@ through which the users can also access its binary data file(s).
      it is not necessarily the same as the original one. It is especially
      true when the object was modified after it is opened.
 
----get(name, level, time)
+---get(name, z, t)
    reads the binary data and returns as a NArray.
 
    ARGUMENTS
    * name (String): name of the variable to read
-   * level (Integer, currently): vertical level to read (0,1,2,...; starting 
+   * z (Integer, currently): vertical level to read (0,1,2,...; starting 
      from 0). Currently only one vertical levels must be chosen, but in the
      future, it is planned to support multiple levels.
-   * time (Integer, currently): time to read (0,1,2,...;  starting 
+   * t (Integer, currently): time to read (0,1,2,...;  starting 
      from 0). Currently only one time must be chosen, but in the
      future, it is planned to support multiple times.
 
@@ -395,13 +387,6 @@ module NumRu
     2
   end
 
-#  def unlimited
-#  def redef
-#  def enddef
-#  def define_mode?
-#  def sync
-#  def def_dim(ary)
-
   def put_att(key,value)
     case key
     when "dset"
@@ -427,8 +412,6 @@ module NumRu
                      :option=>option.to_s, :description=>description.to_s})
   end
 
-#  def def_var_with_dim
-
   def var( varname=nil )
     GrADSVar.new(self,varname)
   end
@@ -444,9 +427,6 @@ module NumRu
     vars
   end
 
-#  def dim
-#  def dims
-
 #  def att
   def get_att( key=nil )
     case(key)
@@ -486,7 +466,7 @@ module NumRu
 
   def to_ctl
     if( !@dimensions[3][:spec] ) 
-      start = generate_starttime(@dimensions[3][:startjd])
+      start = generate_starttime(@dimensions[3][:startdatetime])
       increment = generate_timeincrement(@dimensions[3][:increment],@dimensions[3][:increment_units])
       @dimensions[3][:spec] = "#{start} #{increment}"
     end
@@ -527,87 +507,115 @@ EOS
 
   def put(ary)
 
-    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
-        @options["little_endian"] )
-      raise "endian specification error"
+    if( ary.class == NArrayMiss )
+      raise "UNDEF is not specified" if @undef == nil
+      ary = ary.to_na(@undef)
     end
 
-    if( @options["big_endian"] || @options["cray_32bit_ieee"] )
-      ary = ary.to_na.hton
-    elsif( @options["little_endian"] )
-      ary = ary.to_na.htov
-    elsif( @options["byteswapped"] )
-      ary = ary.to_na.swap_byte
-    end
+    ary = convert_endian_write(ary)
+
+    raise "DSET is not  specified" if @dset == nil
 
-#    if File.exist?(@dset)
-#      print "overwrite "+@dset+" (y/n)? "
-#      ans = gets[0].chr
-#      if ans != "y"
-#        print @dset+" remain unmodified. \n"
-#        return
-#      end
-#    end
-#    putfile = File.open(@dset,"w")
     putfile = File.open(@dset,"wb")
     putfile << ary.to_s
     putfile.close
+
   end
 
-#  def get(name, level, time)
-  def get(name, level, time, lonlat=nil)
+  def get(name, z, t, lonlat=nil)
+
+# t:    [0,1,2,..] : record number
+# time: [0,1/24,2/24,...] days since 00:00Z01jan2000 : Numeric with units
+# date: [00:00Z01jan2000,01:00Z01jan2000,...] : DateTime class
 
     if ( @options["template"] )
-#      time_a = get_dim(@dimensions[3]) # julian day(integer) + time of day
-      time_a = get_dim(@dimensions[3])
-#      date0 = Date.parse(@dimensions[3][:units]).jd # date of the initial record
-      date0 = @dimensions[3][:startjd] # (julian day+time of day) of the initial record
-      time_a = time_a + date0
 
-#      time_d = Date.new1(time_a[time]-0.5) # precise julian day for 00UTC
-      time_d = Date.new1(time_a[time].floor) # julian day(integer)
-      init_year = time_d.year
-      init_month = time_d.month
-      init_day = time_d.day
+      ary_time = get_dim(@dimensions[3])
+      start_date = @dimensions[3][:startdatetime]
+
+      case dimensions[3][:increment_units]
+      when 'years'
+        target_date = start_date >> (ary_time[t]*12)
+      when 'months'
+        target_date = start_date >> ary_time[t]
+      when 'days'
+        target_date = start_date + ary_time[t]
+      when 'hours'
+        target_date = start_date + Rational((ary_time[t]*60).to_i,24*60)
+      when 'minutes'
+        target_date = start_date + Rational(ary_time[t].to_i,24*60)
+      end
+
+      # substitute DSET by the target file name
+      # and determine first record in the target file (="init")
+
+      init_year = target_date.year
+      init_month = target_date.month
+      init_day = target_date.day
+      init_hour = target_date.hour
+      init_min = target_date.min
+
       @dset_r = @dset.dup
-      if ( @dset_r =~ /%y2/ || @dset_r =~ /%y4/)
-        init_month = 1  if (!( @dset_r =~ /%m2/ )&&!( @dset_r =~ /%m1/ ))
-        init_day = 1    if !( @dset_r =~ /%d2/ )
-        @dset_r.gsub!( /%y2/, sprintf("%02d",time_d.year%100) )
-        @dset_r.gsub!( /%y4/, sprintf("%04d",time_d.year) )
+      if ( @dset_r =~ /%y[24]/ )
+        init_month = 1  if !( @dset_r =~ /%m[12]/ )
+        init_day = 1    if !( @dset_r =~ /%d[12]/ )
+        init_hour = 0   if !( @dset_r =~ /%h[12]/ )
+        init_min = 0 if !( @dset_r =~ /%n2/ )
+        @dset_r.gsub!( /%y2/, sprintf("%02d",target_date.year%100) )
+        @dset_r.gsub!( /%y4/, sprintf("%04d",target_date.year) )
       end
-      if ( @dset_r =~ /%m2/ )
-        init_day = 1    if !( @dset_r =~ /%d2/ )
-        @dset_r.gsub!( /%m2/, sprintf("%02d",time_d.month) )
+      if ( @dset_r =~ /%m[12]/ )
+        init_day = 1    if !( @dset_r =~ /%d[12]/ )
+        init_hour = 0   if !( @dset_r =~ /%h[12]/ )
+        init_min = 0 if !( @dset_r =~ /%n2/ )
+        @dset_r.gsub!( /%m1/, sprintf("%d",target_date.month) )
+        @dset_r.gsub!( /%m2/, sprintf("%02d",target_date.month) )
       end
-      if ( @dset_r =~ /%m1/ )
-        init_day = 1    if !( @dset_r =~ /%d2/ )
-        @dset_r.gsub!( /%m1/, sprintf("%d",time_d.month) )
+      if ( @dset_r =~ /%d[12]/ )
+        init_hour = 0   if !( @dset_r =~ /%h[12]/ )
+        init_min = 0 if !( @dset_r =~ /%n2/ )
+        @dset_r.gsub!( /%d1/, sprintf("%d",target_date.day) )
+        @dset_r.gsub!( /%d2/, sprintf("%02d",target_date.day) )
       end
-      if ( @dset_r =~ /%d2/ )
-#        time_d...
-        @dset_r.gsub!( /%d2/, sprintf("%02d",time_d.day) )
+      if ( @dset_r =~ /%h[12]/ )
+        init_min = 0 if !( @dset_r =~ /%n2/ )
+        @dset_r.gsub!( /%h1/, sprintf("%d",target_date.hour) )
+        @dset_r.gsub!( /%h2/, sprintf("%02d",target_date.hour) )
       end
-      time_i = Date.new3(init_year,init_month,init_day).jd
-      init_in_file = (time_a.ge time_i).count_false
-      time_in_file = time - init_in_file
-      start_byte = start_byte(name, level, time_in_file)
-#      @datafile = File.open(@dset_r,"r")
+      if ( @dset_r =~ /%n2/ )
+        @dset_r.gsub!( /%n2/, sprintf("%02d",target_date.min) )
+      end
+
+      init_date = DateTime.new(init_year,init_month,init_day,init_hour,init_min)
+      init_t = 0
+      ary_time[0..t].to_a.each_with_index{ |time, idx| 
+        case dimensions[3][:increment_units]
+        when 'years'
+          before_target_file = ((start_date >> (time*12)) < init_date)
+        when 'months'
+          before_target_file = ((start_date >> time) < init_date)
+        when 'days'
+          before_target_file = ((start_date + time)  < init_date)
+        when 'hours'
+          before_target_file = ((start_date + Rational((time*60).to_i,24*60)) < init_date)
+        when 'minutes'
+          before_target_file = ((start_date + Rational(time.to_i,24*60)) < init_date)
+        end
+        init_t = idx + 1 if before_target_file
+      }
+
+      t_in_target_file = t - init_t
+
+      start_byte = start_byte(name, z, t_in_target_file)
       @datafile = File.open(@dset_r,"rb")
     else
-      start_byte = start_byte(name, level, time)
-#      @datafile = File.open(@dset,"r")
+      start_byte = start_byte(name, z, t)
       @datafile = File.open(@dset,"rb")
     end
 
     @x_len = @dimensions[0][:len]
     @y_len = @dimensions[1][:len]
 
-    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
-        @options["little_endian"] )
-      raise "endian specification error"
-    end
-
     if(lonlat)
       if !(lonlat.is_a?(Array) || lonlat.is_a?(NArray))
         raise "lonlat must be given Array or NArray"
@@ -619,47 +627,49 @@ EOS
       
       if( @map[name][:xytranspose] )
         @datafile.pos = start_byte + @y_len*lon_str*@map[name][:byte]
-        ary = NArray.to_na(@datafile.read(@y_len*(lon_end-lon_str+1)*@map[name][:byte]), 
-                           @map[name][:type], @y_len, lon_end-lon_str+1)
+        readdata = @datafile.read(@y_len*(lon_end-lon_str+1)*@map[name][:byte])
 
-        if( @options["big_endian"] || @options["cray_32bit_ieee"] )
-          ary = ary.ntoh
-        elsif( @options["little_endian"] )
-          ary = ary.vtoh
-        elsif( @options["byteswapped"] )
-          ary = ary.swap_byte
+        if( readdata == nil )
+          raise "File Read Error: #{@datafile.path}, " +
+            "#{@x_len*(lat_end-lat_str+1)*@map[name][:byte]} bytes " +
+            "at #{@datafile.pos} bytes\n" +
+            "specified by #{@ctlfile.path}, #{name} ( z=#{z}, t=#{t} ) (Perhaps there is a mismatch between the control and data files)"
         end
 
+        ary = NArray.to_na(readdata, @map[name][:type], @y_len, lon_end-lon_str+1)
+
+        ary = convert_endian_read(ary)
         ary = ary[lat_str..lat_end,true].transpose
       else
         @datafile.pos = start_byte + @x_len*lat_str*@map[name][:byte]
-        ary = NArray.to_na(@datafile.read(@x_len*(lat_end-lat_str+1)*@map[name][:byte]), 
-                           @map[name][:type], @x_len, lat_end-lat_str+1)
+        readdata = @datafile.read(@x_len*(lat_end-lat_str+1)*@map[name][:byte])
 
-        if( @options["big_endian"] || @options["cray_32bit_ieee"] )
-          ary = ary.ntoh
-        elsif( @options["little_endian"] )
-          ary = ary.vtoh
-        elsif( @options["byteswapped"] )
-          ary = ary.swap_byte
+        if( readdata == nil )
+          raise "File Read Error: #{@datafile.path}, " +
+            "#{@x_len*(lat_end-lat_str+1)*@map[name][:byte]} bytes " +
+            "at #{@datafile.pos} bytes\n" +
+            "specified by #{@ctlfile.path}, #{name} ( z=#{z}, t=#{t} ) (Perhaps there is a mismatch between the control and data files)"
         end
 
+        ary = NArray.to_na(readdata, @map[name][:type], @x_len, lat_end-lat_str+1)
+
+        ary = convert_endian_read(ary)
         ary = ary[lon_str..lon_end,true]
       end
     else
       @datafile.pos = start_byte
-#      ary = NArray.to_na(@datafile.read(@xy*@map[name][:byte]), @map[name][:type],
-      ary = NArray.to_na(@datafile.read(@x_len*@y_len*@map[name][:byte]), @map[name][:type],
-                         @dimensions[0][:len],@dimensions[1][:len])
+      readdata = @datafile.read(@x_len*@y_len*@map[name][:byte])
 
-      if( @options["big_endian"] || @options["cray_32bit_ieee"] )
-        ary = ary.ntoh
-      elsif( @options["little_endian"] )
-        ary = ary.vtoh
-      elsif( @options["byteswapped"] )
-        ary = ary.swap_byte
+      if( readdata == nil )
+        raise "File Read Error: #{@datafile.path}, " +
+          "#{@x_len*(lat_end-lat_str+1)*@map[name][:byte]} bytes " +
+          "at #{@datafile.pos} bytes\n" +
+          "specified by #{@ctlfile.path}, #{name} ( z=#{z}, t=#{t} ) (Perhaps there is a mismatch between the control and data files)"
       end
 
+      ary = NArray.to_na(readdata, @map[name][:type], @dimensions[0][:len],@dimensions[1][:len])
+
+      ary = convert_endian_read(ary)
       ary = ary.transpose if ( @map[name][:xytranspose] )
     end
 
@@ -672,7 +682,6 @@ EOS
 
   def get_dim(dim)
     name = dim[:name]
-    if (name != 't')
       if (dim[:levels])
         var_dim = NArray.to_na(dim[:levels])
       elsif (dim[:start] && dim[:increment] && dim[:len])
@@ -680,38 +689,8 @@ EOS
       else
         raise "cannot define dimension "+name
       end
-#      var_dim = var_dim[0..length-1] if (length)
       var_dim = var_dim[-1..0] if (name == 'y' && @options["yrev"])
       var_dim = var_dim[-1..0] if (name == 'z' && @options["zrev"])
-    else # time
-      if (dim[:levels])
-        var_dim = NArray.to_na(dim[:levels])
-      elsif(dim[:start] && dim[:increment] && dim[:len] && dim[:increment_units])
-#        var_dim = NArray.sfloat(dim[:len])
-        var_dim = NArray.float(dim[:len])
-        case dim[:increment_units]
-        when 'day'
-          var_dim = var_dim.indgen!*dim[:increment]+dim[:start]
-        when 'month'
-          date0 = dim[:startjd] # (julian day+time of day) of the initial record
-          var_dim[0] = dim[:start]
-          for i in 1..dim[:len]-1
-            var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment] ).jd - date0
-          end
-        when 'year'
-          date0 = dim[:startjd] # (julian day+time of day) of the initial record
-          var_dim[0] = dim[:start]
-          for i in 1..dim[:len]-1
-            var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment]*12 ).jd - date0
-          end
-        else
-          raise "invalid units: "+dim[:increment_units]
-        end
-      else
-        raise "cannot define dimension "+name
-      end
-#      var_dim = var_dim[0..length-1] if (length)
-    end
     return var_dim
   end
 
@@ -848,11 +827,11 @@ EOS
             # idim = 3 --- time
             if (dim[:flag] =~ /LINEAR/i)
               start,increment= spec.split
-              year,mon,day,hour,min = parse_starttime(start)
               dim[:start] = 0.0
-              dim[:startjd] = Date.new(year,mon,day).jd + hour/24.0 + min/1440.0
+              dim[:startdatetime] = parse_starttime(start)
               dim[:increment],dim[:increment_units] = parse_timeincrement(increment)
-              dim[:units] = sprintf("days since #{year}-%02i-%02i %02i:%02iZ",mon,day,hour,min)
+              dim[:units] = dim[:increment_units]+" since "+dim[:startdatetime].strftime("%Y-%m-%d %H:%MZ")
+
             else
               raise "invalid dimension flag(only LINEAR is available for time)"
             end
@@ -862,7 +841,6 @@ EOS
           raise "Invalid line: "+line
         end
       when /^\s*VARS/i
-#        total_levs=0
         total_lev_bytes=0
         while ( vline = @ctlfile.gets )
           case(vline)
@@ -906,24 +884,15 @@ EOS
           end
         end
         @map=Hash.new
-#        cum_lev = 0
         cum_lev_bytes = 0
         @variables.each{|i|
           varname = i[:name]
-#          case (i[:option])
-#          when /^0/,/^99/,/^-1,30/
-            @map[varname]={:offset=>@fileheader_len,:nlev=>i[:nlev],
-#              :start=>cum_lev,:zstep=>1,:tstep=>total_levs}
-              :start=>cum_lev_bytes,:zstep=>i[:byte],:tstep=>total_lev_bytes, 
-              :type=>i[:type],:byte=>i[:byte]}
-            if i[:option] =~ /^-1,30/
-              @map[varname][:xytranspose]=true
-            end
-#            p "## test - #{varname}",@map[varname]
-#          else
-#            raise "invalid or unsupported variable placement option: "+i[:option]
-#          end
-#          cum_lev += i[:nlev]
+          @map[varname]={:offset=>@fileheader_len,:nlev=>i[:nlev],
+            :start=>cum_lev_bytes,:zstep=>i[:byte],:tstep=>total_lev_bytes, 
+            :type=>i[:type],:byte=>i[:byte]}
+          if i[:option] =~ /^-1,30/
+            @map[varname][:xytranspose]=true
+          end
           cum_lev_bytes += i[:nlev]*i[:byte]
         }
       end
@@ -987,12 +956,9 @@ EOS
         raise "Cannot convert hour or time into interger: "+stime
       end
     else
-      #shour = '00'
-      #smin = '00'
       hour = 0
       min = 0
     end
-    timeofday = min/1440.0 + hour/24.0
 
     if ( /(\d*)(\w\w\w)(\d\d\d\d)/ =~ sdate )
       sday = $1
@@ -1004,22 +970,17 @@ EOS
         else
           day = sday.to_i
         end
-#        day = sday.to_i
         year = syear.to_i
         mon=['jan','feb','mar','apr','may','jun','jul','aug','sep',
           'oct','nov','dec'].index(smon.downcase) + 1
-        date = Date.new(year,mon,day)
       rescue StandardError, NameError
-#        raise $!+"\nCould not parse the date string: "+sdate
         raise "Could not parse the date string: "+sdate+"\n"+$!
       end
     else
       raise "The date part must be [dd]mmmyyyy, but what was given is: "+sdate
     end
 
-#    julian_day = date.jd + timeofday
-#    return julian_day
-    return year,mon,day,hour,min
+    return DateTime.new(year,mon,day,hour,min)
   end
 
   def parse_timeincrement(string)
@@ -1028,24 +989,28 @@ EOS
       sunits = $2
       case sunits
       when /mn/i
-        fact = 1.0/1440.0   # factor to convert into days
-        units = 'day'
+#        fact = 1.0/1440.0   # factor to convert into days
+#        units = 'days'
+        fact = 1.0
+        units = 'minutes'
         increment = sincrement * fact
       when /hr/i
-        fact = 1.0/24.0
-        units = 'day'
+#        fact = 1.0/24.0
+#        units = 'days'
+        fact = 1.0
+        units = 'hours'
         increment = sincrement * fact
       when /dy/i
         fact = 1.0
-        units = 'day'
+        units = 'days'
         increment = sincrement * fact
       when /mo/i
         fact = 1.0
-        units = 'month'
+        units = 'months'
         increment = sincrement * fact
       when /yr/i
         fact = 1.0
-        units = 'year'
+        units = 'years'
         increment = sincrement * fact
       else
         raise "invalid units: "+sunits
@@ -1060,15 +1025,9 @@ EOS
     a>b ? a : b
   end
 
-  def generate_starttime(startjd)
-    date = Date.jd(startjd)
-    year, mon, day = date.year, date.mon, date.day
-    hour = startjd*24%24
-    min = startjd*1440%60
-
-    mons = ['jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec']
-
-    return sprintf("%02d:%02dZ%02d",hour,min,day)+mons[mon-1]+sprintf("%04d",year)
+  def generate_starttime(datetime)
+    mon = Date::ABBR_MONTHNAMES[datetime.mon]
+    return datetime.strftime("%H:%MZ%d#{mon}%Y")
   end
 
   def generate_timeincrement(increment,units)
@@ -1112,27 +1071,43 @@ EOS
     return sprintf("%2d",sincrement)+sunits
   end
 
-  def initialize_new # obsolete
+  def convert_endian_read(ary)
 
-    raise "Sorry, this method is far from completed"
+    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
+        @options["little_endian"] )
+      raise "endian specification error"
+    end
 
-    @dims = []
-    @variables = []
-    
-    #<attributes>
-    @dset = nil
-    @title = nil
-    @undef = nil
-    @fileheader_len = 0
-    @options = []
-    
-    #<internal control parameters>
-    @define_mode = true
-    @ctl_dumped = false
+    if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+      ary = ary.ntoh
+    elsif( @options["little_endian"] )
+      ary = ary.vtoh
+    elsif( @options["byteswapped"] )
+      ary = ary.swap_byte
+    end
+
+    ary
   end
-  
- end
 
+  def convert_endian_write(ary)
+
+    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
+        @options["little_endian"] )
+      raise "endian specification error"
+    end
+
+    if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+      ary = ary.hton
+    elsif( @options["little_endian"] )
+      ary = ary.htov
+    elsif( @options["byteswapped"] )
+      ary = ary.swap_byte
+    end
+
+    ary
+  end
+
+ end
 
  class GrADSVar
 
@@ -1343,10 +1318,6 @@ EOS
         end
       end
 
-#      idx_x = NArray.int(@ctl.dimensions[0][:len]).indgen!
-#      idx_y = NArray.int(@ctl.dimensions[1][:len]).indgen!
-#      idx_z = NArray.int(@ctl.dimensions[2][:len]).indgen!
-#      idx_t = NArray.int(@ctl.dimensions[3][:len]).indgen!
       idx_x = NArray.int(@shape[0]).indgen!
       idx_y = NArray.int(@shape[1]).indgen!
       idx_z = NArray.int(@shape[2]).indgen!
@@ -1357,11 +1328,11 @@ EOS
       str_t = idx_t[h_sta[3]]; end_t = idx_t[h_end[3]]
 
       na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-      for time in str_t..end_t
-        for lev in str_z..end_z
-          na_xy = @ctl.get(@varname,lev,time,[str_x,end_x,str_y,end_y])
+      for t in str_t..end_t
+        for z in str_z..end_z
+          na_xy = @ctl.get(@varname,z,t,[str_x,end_x,str_y,end_y])
           mask = ( na_xy.ne @ctl.undef )
-          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy,mask)
+          na[true,true,z-str_z,t-str_t] = NArrayMiss.to_nam(na_xy,mask)
         end
       end
     end
