@@ -350,19 +350,29 @@ These methods returns a NArray (not a VArray).
 	  ary = @varray.ary
 	  slicer = @mapping.slicer
 	  if ary.is_a?(NArray) || ary.is_a?(NArrayMiss)
-	    # interpret Hash slicers for NArray/NArrayMiss
-	    slicer.collect!{ |sl|
+	    # interpret Hash slicers for NArray/NArrayMiss 
+	    #   -- this part would not be needed if NArray supported it.
+	    new_slicer = Array.new
+	    for idx in 0...slicer.length
+	      sl = slicer[idx]
 	      if sl.is_a?(Hash)
 		range, step = sl.to_a[0]
+		dim_len = ary.shape[idx]
 		first = range.first
+		first += dim_len if first<0
 		last = range.last
+		last += dim_len if last<0
+		if first<0 || first>=dim_len || last<0 || last>=dim_len || step==0
+		  raise "slicer #{slicer.inspect} for dim #{idx} is invalid (dim_len=#{dim_len})"
+		end
 		step = -step if ( (last-first)*step < 0 )
 		length = (last-first) / step + 1
-		first + step * NArray.int(length).indgen!
+		new_slicer[idx] = first + step * NArray.int(length).indgen!
 	      else
-		sl
+		new_slicer[idx] = sl
 	      end
-	    }
+	    end
+	    slicer = new_slicer
 	  end
 	  ary[*slicer]
 	else
@@ -381,17 +391,21 @@ These methods returns a NArray (not a VArray).
 
       def replace_val(narray)
 	narray = __check_ary_class(narray)
-	if narray.shape != shape
+	if self.class != VArray
+	  raise "replace_val to #{self.class} is disabled. Use val= instead"
+	end
+	if narray.shape != shape 
 	  raise "shape of the argument (#{narray.shape.inspect}) !="+
-          " self.shape (#{shape.inspect})"
+	    " self.shape (#{shape.inspect})"
 	end
-	if self.class == VArray
-	  @ary = narray
-	  self
-	else
-	  # subclass --> VArray
-	  VArray.new(narray, attr_copy, name)
+	@ary = narray
+	if @mapping
+	  # to non subset
+	  @name = @varray.name
+	  @attr = @varray.attr_copy
+	  @mapping = @varray = nil
 	end
+	self
       end
 
       def ntype
