@@ -3,33 +3,21 @@
 * Support a temporay attribute (@tmp_attr = Attribute.new)
 =end
 
-#require "numru/netcdf"
 require "numru/gphys/grads_gridded"
 require "numru/gphys/varray"
-#require "numru/gphys/attributenetcdf"
-
 
 module NumRu
 
-#   class VArrayNetCDF < VArray
    class VArrayGrADS < VArray
 
       ## < initialization redefined > ##
 
-#      def initialize(aNetCDFVar)
       def initialize(aGrADSVar)
-#         @name = aNetCDFVar.name
          @name = aGrADSVar.name
          @mapping = nil
          @varray = nil
-#         raise ArgumentError,"Not a NetCDVAr" if ! aNetCDFVar.is_a?(NetCDFVar)
          raise ArgumentError,"Not a GrADSVar" if ! aGrADSVar.is_a?(GrADSVar)
-#         @ary = aNetCDFVar
          @ary = aGrADSVar
-#         @ary.extend(NetCDFVarDeferred)
-#         @ary.file.extend(NetCDFDeferred)
-#         @attr = AttributeNetCDF.new(aNetCDFVar)
-#         @attr = Attribute.new
          @attr = aGrADSVar.attr
       end
 
@@ -54,9 +42,19 @@ module NumRu
            rank = gphys.data.rank
            raise ArgumentError, "only 4D data is supported" if rank != 4
 
-           for i in 0..2
+           for i in 0..2 # lon, lat, lev
              c = ""
-             gphys.axis(i).pos.val.each{|j| c += (j.to_s + " ")}
+             if   ( i==1 && (gphys.coord(i).length > 1) && 
+                 (gphys.coord(i).val[1]-gphys.coord(i).val[0] < 0) )
+               file.put_att("yrev",true)
+               gphys.axis(i).pos.val[-1..0].each{|j| c += (j.to_s + " ")}
+             elsif( i==2 && (gphys.coord(i).length > 1) && 
+                 (gphys.coord(i)[1].val-gphys.coord(i)[0].val > 0) )
+               file.put_att("zrev",true)
+               gphys.axis(i).pos.val[-1..0].each{|j| c += (j.to_s + " ")}
+             else
+               gphys.axis(i).pos.val.each{|j| c += (j.to_s + " ")}
+             end
              file.dimensions[i] = {
                :len=>gphys.axis(i).pos.length, 
                :flag=> "LEVELS", 
@@ -65,16 +63,41 @@ module NumRu
            end
 
            i = 3 # time
-             c = Date.jd(gphys.axis(i).pos.val[0]).strftime("%d%b%Y") + " <increment>"
+             tunit = Units.new(gphys.coord(i).get_att("units"))
+             basedate = Date.new(1950,1,1).jd # temporary base date
+             baseunits = Units.new("days since 1950-01-01 0:0:0 +0:00")
+             days_since_basedate = tunit.convert(gphys.coord(i).val[0],baseunits)
+             startjd = basedate + days_since_basedate
+
+             if( gphys.coord(i).length > 1 )
+               increment = gphys.coord(i)[1].val - gphys.coord(i)[0].val
+             else
+               increment = 1.0 # anything is possible
+             end
+
+#             c = Date.jd(gphys.axis(i).pos.val[0]).strftime("%d%b%Y") + " <increment>"
              file.dimensions[i] = {
                :len=>gphys.axis(i).pos.length, 
                :flag=> "LINEAR", 
-               :spec=>c
+#               :spec=>c
+               :startjd=> startjd, 
+               :increment=> increment, 
+               :increment_units=> "day"
              }
            # end
 
-           file.def_var(gphys.name, gphys.data.get_att("nlev"), 
-                        99, gphys.data.get_att("long_name") )
+#           file.def_var(gphys.name, gphys.data.get_att("nlev"), 
+#                        99, gphys.data.get_att("long_name") )
+           nlev1 = gphys.coord(2).length
+           if( nlev = gphys.data.get_att("nlev") )
+             nlev = nlev.to_i
+             if( (nlev1 != nlev) && !(nlev1==1 && nlev==0) )
+               raise "inconsistent attribute nlev (#{nlev} and #{nlev1})"
+             end
+           else
+             nlev = nlev1
+           end
+           file.def_var(gphys.name, nlev, 99, gphys.data.get_att("long_name") )
 
            ctlfile = File.open(file.path,"w")
            ctlfile << file.to_ctl
