@@ -341,6 +341,10 @@ module NumRu
       contour = opt.delete(:contour) 
       tone = opt.delete(:tone) 
       color_bar = opt.delete(:color_bar)
+      color_bar_options = opt.delete(:color_bar_options)
+      if color_bar_options && !color_bar_options.is_a?(Hash)
+	raise ":color_bar_options must be a Hash"
+      end
 
       if @set[:fig] && (itr=@set[:fig]['itr']) && itr>=10
 	lon = gphys.coord(0)
@@ -366,7 +370,11 @@ module NumRu
         GGraph::contour(gphys, newfrm, opt)
         newfrm = false
       end
-      GGraph::color_bar() if tone && color_bar
+      if tone && color_bar
+	cbopt = color_bar_options || Hash.new
+	cbopt['log'] = true if opt['log']
+	GGraph::color_bar(cbopt)
+      end
       newfrm
     end
 
@@ -507,6 +515,30 @@ if __FILE__ == $0
 
     viz.execute
 
+  when 6
+
+    viz = NumRu::VizShot.new(:iwidth=>700,:iheight=>500,:xdiv=>2,:ydiv=>2)
+    viz.set_fig('viewport'=>[0.2,0.8,0.27,0.6])
+    viz.plot( :method => :tone_cont, :newfrm=>true,
+	      :file => "../../testdata/T.jan.nc", 
+	      :var=> "T", :cut => {'level'=>1000},
+	     :color_bar=>true, :color_bar_options=>{'vlength'=>0.25})
+    viz.plot( :method => :tone_cont, :newfrm=>true,
+	      :file => "../../testdata/T.jan.nc", 
+	      :var=> "T", :cut => {'level'=>1000},
+	      :color_bar=>true, :color_bar_options=>{'labelintv'=>1})
+    viz.plot( :method => :tone_cont, :newfrm=>true,
+	      :file => "../../testdata/T.jan.nc", 
+	      :var=> "T", :cut => {'level'=>1000},
+	      :color_bar=>true, 
+              :color_bar_options=>{'landscape'=>true, 'labelintv'=>1,
+               'vlength'=>0.6})
+
+    path = viz.dump_code
+    print "\n!!! Code written in #{path} !!!\n\n"
+
+    viz.execute
+
   end
 
 end
